@@ -2,36 +2,40 @@ package Plack::App::URLMux;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use parent qw(Plack::Component);
 use Carp qw(croak);
 
-use constant MAX_FINE        => 65535; # if URL will contain more than this number - increment it :)
+use constant MAX_FINE        => 65535; # if URL will contain url-path segments more than this number - increment it :)
 
 # map[] structure constants
-use constant _keys           => 0; # hash of url subpath => map[]
+use constant _keys           => 0; # hash of url-path segments => map[]
 use constant _level          => 1; # level of map tree
 use constant _quant          => 2; # rule for quantifier at branch, this value sensitive for _named array
 use constant _app            => 3; # reference to app on this leaf, undefined if does not mounted any
-use constant _named          => 4; # reference to map[], if subpath has named parameter then mapping searching in it
+use constant _named          => 4; # reference to map[], if url-path has named parameter then mapping searching in it
 use constant _names          => 5; # array of [parameter, index], if app has parameters in the url format
 use constant _params         => 6; # array of pairs name=>value of input parameters specified at map
-use constant _quants         => 7; # rule for quantifiers for mounted app
+use constant _quants         => 7; # array of quantifiers of each url-path segments for mounted app
 
+# _quant array constants
 use constant _quant_n        => 0; # start range quanifier
 use constant _quant_m        => 1; # end range qunatifier, -1 unknown maximum as possible
-use constant _quant_r        => 2; # rest of posible search path, -1 unknown
+use constant _quant_r        => 2; # rest of posible search path, -1 unknown, unusefull, FIXME delete this
 use constant _quant_h        => 3; # greedy/lazy flag for search path and gather subpathes for parameter, 0 lazy, 1 greedy
 
+# _params array constants
 use constant _param_name     => 0;
 use constant _param_index    => 1;
 
+# matched array constants
 use constant _match_map      => 0;
 use constant _match_length   => 1;
 use constant _match_fine     => 2;
 use constant _match_matching => 3;
 
+# matching array constants
 use constant _matching_index => 0;
 use constant _matching_fine  => 1;
 
@@ -43,6 +47,14 @@ sub map {
     my $app      = shift;
     my $input_params = \@_;
 
+
+    if ($location eq '') {
+        croak "location 'not found' has already mount app"
+            if exists $self->{_mapping_not_found} and defined $self->{_mapping_not_found};
+        $self->{_mapping_not_found} = $app;
+        return;
+    }
+
     my $host = '*';  #any host
     if ($location =~ m/^https?:\/\/(.*?)(\/.*)/o) {
         $host     = $1;
@@ -56,7 +68,7 @@ sub map {
     my @paths = split('/', $location);
 
     my ($index, $params, $quants, $quant) = (0, [], [], undef);
-    my $map = $self->{_mapping}->{$host} ||= [{}, $index, [1, 1]];  # Zero mapping for URL '/'
+    my $map = $self->{_mapping}->{$host} ||= [{}, $index, [1, 0]];  # Zero mapping for URL '/'
     $index++;
     foreach my $path (@paths) {
         my $r = @paths - $index;
@@ -94,7 +106,7 @@ sub map {
             ($n, $m) = ($n, ($m ? $m : -1)     ) if $quant eq ',';
             ($n, $m) = ($n = ($n ? $n : 1), $n ) if $quant eq '';
 
-            #check _named now named must be array but we must check what check
+            #check _named now named must be array but we must check
             push(@$params, [$name, $index]);
             push(@$quants, [$n, $m, $r, $h]);
 
@@ -120,7 +132,6 @@ sub map {
     croak "/$location has already mount app"
         if defined $map->[_app];
 
-
     @$map[_app,_params,_names,_quants] = ($app, $input_params, @$params ? $params : undef, $quants);
 
 }
@@ -141,7 +152,7 @@ sub call {
         = @{$env}{qw( HTTP_HOST SERVER_NAME SCRIPT_NAME PATH_INFO )};
 
     #FIXME possible BUG
-    # there is cases when port is not the same in HTTP_HOST and SERVER_PORT?
+    # is there cases when port is not the same in HTTP_HOST and SERVER_PORT?
     if ($http_host and $env->{SERVER_PORT}) {
         $http_host =~ s/:\d+$//o;
     }
@@ -162,8 +173,11 @@ sub call {
             )
         );
 
-    return [404, [ 'Content-Type' => 'text/plain' ], [ "Not Found" ]]
-        unless $matched;
+    unless ($matched) {
+        return [404, [ 'Content-Type' => 'text/plain' ], [ "Not Found" ]]
+            unless $self->{_mapping_not_found};
+        return $self->{_mapping_not_found}->($env);
+    }
 
     #now we have first matched rule and match path, we need fill params if they exists
 
@@ -396,6 +410,7 @@ specified when mount app
   $urlmap->map("/foo" => $app);
   $urlmap->map("http://bar.example.com/" => $another_app);
   $urlmap->map("/foo/:name/bar" => $app2);
+  $urlmap->map('' => $app_not_found);
 
 Maps URL an absolute URL to a PSGI application. Module splits url path by
 '/' delimeter and add it into builded search tree structure.
@@ -408,6 +423,9 @@ C</foo/> or C</foo/bar> but it I<won't> match with C</foox>.
 Mapping URLs with host names is also possible, and in that case the URL
 mapping works like a virtual host.
 
+Also possible handle 404 error by outer application that handles 'Not found'
+event by mapping the application to an empty location.
+
 =item mount
 
 Alias for C<map>.
